from setuptools import find_packages, setup

setup(
    name="any_serde",
    version="0.2.16",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    package_data={
        "": ["*py.typed"],
    },
    data_files=[
        (
            "any_serde/typescript",
            [
                "any_serde/typescript/bool_typedef.ts.jinja2",
                "any_serde/typescript/dataclass_typedef.ts.jinja2",
                "any_serde/typescript/float_typedef.ts.jinja2",
                "any_serde/typescript/int_typedef.ts.jinja2",
                "any_serde/typescript/list_typedef.ts.jinja2",
                "any_serde/typescript/literal_typedef.ts.jinja2",
                "any_serde/typescript/none_typedef.ts.jinja2",
                "any_serde/typescript/nonetype_typedef.ts.jinja2",
                "any_serde/typescript/string_typedef.ts.jinja2",
                "any_serde/typescript/union_typedef.ts.jinja2",
            ],
        ),
    ],
)



import types
from typing import (
    Any,
    List,
    Sequence,
    Set,
    Type,
    TypeVar,
    Union,
    get_args,
    get_origin,
)
from any_serde.common import (
    InvalidSerializationException,
    InvalidDeserializationException,
    JSON,
    resolve_newtypes,
    truncate_str,
)


T_Any = TypeVar("T_Any")


def _get_union_args(type_: Type[T_Any]) -> Sequence[Type[Any]]:
    type_origin_nullable = get_origin(type_)

    assert type_origin_nullable is not None, f"Calling union serde on non-union type: {type_}"

    type_origin = resolve_newtypes(type_origin_nullable)
    type_args = [resolve_newtypes(type_arg) for type_arg in get_args(type_)]

    assert type_origin in (Union, types.UnionType), f"Calling union serde on non-union type: {type_}"

    seen: Set[Type[Any]] = set()
    deduped_type_args: List[Type[Any]] = []
    for type_arg in type_args:
        if type_arg in seen:
            continue
        deduped_type_args.append(type_arg)
        seen.add(type_arg)

    return deduped_type_args


def from_data(
    type_: Type[T_Any],
    data: JSON,
) -> T_Any:
    type_args = _get_union_args(type_)

    from any_serde import serde

    for union_arg in type_args:
        try:
            return serde.from_data(union_arg, data)
        except InvalidDeserializationException:
            pass

    raise InvalidDeserializationException("Could not deserialize to any union option.")


def to_data(
    type_: Type[T_Any],
    item: T_Any,
) -> JSON:
    type_args = _get_union_args(type_)

    from any_serde import serde

    for union_arg in type_args:
        try:
            return serde.to_data(union_arg, item)
        except InvalidSerializationException:
            pass

    raise InvalidSerializationException(f"No union option for {type_} matches {truncate_str(str(item))}")



from pathlib import Path
from jinja2 import Template


TYPESCRIPT_MODULE_DIR = Path(__file__).resolve().parent


def load_template(path: Path) -> Template:
    with path.open("rt") as fin:
        return Template(fin.read())
